MODULE calc 
	IMPLICIT NONE
	PRIVATE
	PUBLIC :: area, cuboid, it_fak, it_dreieck, rek_fak, rek_bin, rek_dreieck, &
				& OPERATOR(.HOCH.), OPERATOR(.UEBER.), quad, istPrimzahl
	
	INTERFACE OPERATOR (.UEBER.)
		MODULE PROCEDURE it_bin
	END INTERFACE
	
	INTERFACE OPERATOR (.HOCH.)
		MODULE PROCEDURE potenz 
	END INTERFACE
	
	
CONTAINS 

	FUNCTION area(d)
		REAL, INTENT(IN) :: d
		REAL :: area, pi = 3.14159265359
		area = pi*d**2/4								!wegen d = 2*r (r Radius)
	END FUNCTION

	SUBROUTINE cuboid(a, b, h, v, f)
			REAL, INTENT(IN) :: a, b, h
			REAL, INTENT(OUT) :: v, f 
			v = a*b*h !Volumenberechnung
			f = 2*(a*b + a*h + b*h) !Flächenberechnung
	END SUBROUTINE
	
	FUNCTION it_fak(n) 
		INTEGER, INTENT(IN) :: n 
		INTEGER :: it_fak, i 
		it_fak = 1
		DO i=1,n 
			it_fak = it_fak*i
		END DO
	END FUNCTION

	FUNCTION it_dreieck(n)
		INTEGER, INTENT(IN) :: n 
		INTEGER :: it_dreieck, i 
		it_dreieck = 0
		DO i=1,n 
			it_dreieck = it_dreieck + i 
		END DO 
	END FUNCTION
	
	FUNCTION it_bin(n,k)
		INTEGER, INTENT(IN) :: n, k 
		INTEGER :: it_bin
		it_bin = it_fak(n) / (it_fak(k)*it_fak(n-k))
	END FUNCTION
	
	RECURSIVE FUNCTION rek_fak(n) RESULT(res)
		INTEGER, INTENT(IN) :: 	n
		INTEGER :: res
		IF(n==0) THEN
			res = 1
		ELSE 
			res = rek_fak(n-1)*n 
		END IF
	END FUNCTION

	RECURSIVE FUNCTION 	rek_dreieck(n) RESULT(res)
		INTEGER, INTENT(IN) :: n 
		INTEGER :: res 
		IF(n==0) THEN 
			res = 0 
		ELSE 
			res = rek_dreieck(n-1) + n 
		END IF 
	END FUNCTION

	RECURSIVE FUNCTION rek_bin(n,k) RESULT(res)
		INTEGER, INTENT(IN) :: n, k 
		INTEGER :: res 
		IF(k>n) THEN 
			res = 0
		ELSE IF(n==k .OR. k==0) THEN 
			res = 1
		ELSE 
			res = rek_bin(n-1,k-1) + rek_bin(n-1,k) 
		END IF 
	END FUNCTION
	
	RECURSIVE FUNCTION potenz(n,e) RESULT(res)
		INTEGER, INTENT(IN) :: n, e 
		INTEGER :: res 
		IF(e==0) THEN 
			res = 1 
		ELSE 
			res = potenz(n,e-1) * n 
		END IF
	END FUNCTION
	
	RECURSIVE FUNCTION quad(n) RESULT(res)
		INTEGER, INTENT(IN) :: n
		INTEGER :: res 
		IF(n==1) THEN 
			res = 1
		ELSE 
			res = quad (n-1) + 2*n - 1
		END IF 
	END FUNCTION
	
	RECURSIVE FUNCTION istPrimzahl(p,z) RESULT(res)
		INTEGER, INTENT(IN) :: p, z
		LOGICAL :: res 
		IF(z==1) THEN 
			res = .TRUE.
		ELSE IF(MOD(p,z)==0) THEN 
			res = .FALSE.
		ELSE 
			res = istPrimzahl(p,z-1)
		END IF 
	END FUNCTION


END MODULE 

PROGRAM taschenrechner2
	USE calc
	IMPLICIT NONE

	REAL :: h, x, y, z, v, f 
	INTEGER :: n, k 
	CHARACTER :: c

	DO
		WRITE(*,*)
		WRITE(*,*) 'Please enter calculation operation '
		WRITE(*,*) '(+,-,*,:, "c" for cirlce area, "p" for prism, "f" for factorial, "t" for triangular number, '
		WRITE(*,*) '"b" for binomial coefficient, "h" for the power of an exponent, "z" for square number, '
		WRITE(*,*) '"r" for prime number, "q" for quit):'
		READ(*,*) c
		WRITE(*,*)

		IF(c == '+'.OR.c=='-'.OR.c=='*'.OR.c==':') THEN
			WRITE(*,*) 'Please enter real values for x and y:'
			READ(*,*) x, y
		ELSE IF(c == 'c') THEN
			WRITE(*,*) 'Please enter real value for diameter:'
			READ(*,*) x
		ELSE IF(c == 'p') THEN
			WRITE(*,*) 'Please enter real values for a, b and height h:'
			READ(*,*) x, y, h
		ELSE IF(c=='f') THEN 
			WRITE(*,*) 'Please enter integer value for n!'
			READ(*,*) n 
		ELSE IF(c=='t') THEN 
			WRITE(*,*) 'Please enter integer value for n for the triangular number'
			READ(*,*) n
		ELSE IF(c == 'b') THEN
			WRITE(*,*) 'Please enter real values for n and k for the binomial coefficent:'
			READ(*,*) n,k 
		ELSE IF(c == 'h') THEN 
			WRITE(*,*) 'Please enter integer values for n and k for n^k: '
			READ(*,*) n, k 
		ELSE IF(c == 'z') THEN 
			WRITE(*,*) 'Please enter integer value for the square number: '
			READ(*,*) n 
		ELSE IF(c == 'r') THEN 
			WRITE(*,*) 'Please enter integer value to check if it is a prime number: '
			READ(*,*) n 
		ELSE IF(c == 'q') THEN
			EXIT
		ELSE
			WRITE(*,*) 'No permitted calculation operation entered.'
		END IF

		SELECT CASE(c)
			CASE('+')
				z = x + y
				WRITE(*,*) x ,'+', y, '=', z
			CASE('-')
				z = x - y
				WRITE(*,*) x ,'-', y, '=', z
			CASE('*')
				z = x*y
				WRITE(*,*) x ,'*', y, '=', z
			CASE(':')
				IF (y/=0) THEN
					z = x/y
					WRITE(*,*) x ,':', y, '=', z
				ELSE
					WRITE(*,*) 'Dividing by zero is not permitted.'
				END IF
			CASE('c')
				WRITE(*,*) 'The area is', area(x)
			CASE('p')
				CALL cuboid(x, y, h, v, f)
				WRITE(*,*) 'Volume of cuboid: ', v 
				WRITE(*,*) 'Area of cuboid: ', f 
			CASE('f')
				WRITE(*,*) 'The factorial(iterativ) is', it_fak(n)
				WRITE(*,*) 'The factorial(recursive) is', rek_fak(n)
			CASE('t')
				WRITE(*,*) 'The triangular number(iterativ) is', it_dreieck(n)
				WRITE(*,*) 'The triangular number(recursive) is', rek_dreieck(n)
			CASE('b')
				!WRITE(*,*) 'The binomial coefficient(iterativ) is', it_bin(n,k)
				!WRITE(*,*) 'The binomial coefficient(recursive) is', rek_bin(n,k)
				WRITE(*,*) 'The binomial coefficent is ', n .UEBER. k 
			CASE('h')
				WRITE(*,*) 'Die Potenz ist ', n .HOCH. k 
			CASE('z') 
				WRITE(*,*) 'The square number is ', quad(n) 
			CASE('r') 
				WRITE(*,*) 'Prime number? ', istPrimzahl(n,n-1)  
			CASE DEFAULT
				EXIT
		END SELECT
	END DO

END PROGRAM taschenrechner2
